@@ -1,4 +1,4 @@
-/* Copyright (c) 2002, 2003, 2004  Eric B. Weddington
+/* Copyright (c) 2002, 2003, 2004, 2005, 2006  Eric B. Weddington
    All rights reserved.
 
    Redistribution and use in source and binary forms, with or without
@@ -26,10 +26,12 @@
   ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
   POSSIBILITY OF SUCH DAMAGE. */
 
+/* $Id: boot.h,v 1.20.2.3 2006/03/09 21:38:38 joerg_wunsch Exp $ */
+
 #ifndef _AVR_BOOT_H_
 #define _AVR_BOOT_H_    1
 
-/** \defgroup avr_boot Bootloader Support Utilities
+/** \defgroup avr_boot <avr/boot.h>: Bootloader Support Utilities
     \code
     #include <avr/io.h>
     #include <avr/boot.h>
@@ -188,8 +190,9 @@
         "sts %0, %1\n\t"                         \
         "spm\n\t"                                \
         "clr  r1\n\t"                            \
-        : "=m" (__SPM_REG)                       \
-        : "r" ((uint8_t)__BOOT_PAGE_FILL),       \
+        :                                        \
+        : "i" (_SFR_MEM_ADDR(__SPM_REG)),        \
+          "r" ((uint8_t)__BOOT_PAGE_FILL),       \
           "r" ((uint16_t)address),               \
           "r" ((uint16_t)data)                   \
         : "r0", "r30", "r31"                     \
@@ -207,8 +210,9 @@
         ".word 0xffff\n\t"                       \
         "nop\n\t"                                \
         "clr  r1\n\t"                            \
-        : "=m" (__SPM_REG)                       \
-        : "r" ((uint8_t)__BOOT_PAGE_FILL),       \
+        :                                        \
+        : "i" (_SFR_MEM_ADDR(__SPM_REG)),        \
+          "r" ((uint8_t)__BOOT_PAGE_FILL),       \
           "r" ((uint16_t)address),               \
           "r" ((uint16_t)data)                   \
         : "r0", "r30", "r31"                     \
@@ -225,9 +229,10 @@
         "sts %0, %2\n\t"                         \
         "spm\n\t"                                \
         "clr  r1\n\t"                            \
-        : "=m" (__SPM_REG),                      \
-          "=m" (RAMPZ)                           \
-        : "r" ((uint8_t)__BOOT_PAGE_FILL),       \
+        :                                        \
+        : "i" (_SFR_MEM_ADDR(__SPM_REG)),        \
+          "i" (_SFR_MEM_ADDR(RAMPZ)),            \
+          "r" ((uint8_t)__BOOT_PAGE_FILL),       \
           "r" ((uint32_t)address),               \
           "r" ((uint16_t)data)                   \
         : "r0", "r30", "r31"                     \
@@ -241,8 +246,9 @@
         "movw r30, %2\n\t"                       \
         "sts %0, %1\n\t"                         \
         "spm\n\t"                                \
-        : "=m" (__SPM_REG)                       \
-        : "r" ((uint8_t)__BOOT_PAGE_ERASE),      \
+        :                                        \
+        : "i" (_SFR_MEM_ADDR(__SPM_REG)),        \
+          "r" ((uint8_t)__BOOT_PAGE_ERASE),      \
           "r" ((uint16_t)address)                \
         : "r30", "r31"                           \
     );                                           \
@@ -257,8 +263,9 @@
         "spm\n\t"                                \
         ".word 0xffff\n\t"                       \
         "nop\n\t"                                \
-        : "=m" (__SPM_REG)                       \
-        : "r" ((uint8_t)__BOOT_PAGE_ERASE),      \
+        :                                        \
+        : "i" (_SFR_MEM_ADDR(__SPM_REG)),        \
+          "r" ((uint8_t)__BOOT_PAGE_ERASE),      \
           "r" ((uint16_t)address)                \
         : "r30", "r31"                           \
     );                                           \
@@ -272,9 +279,10 @@
         "sts  %1, %C3\n\t"                       \
         "sts %0, %2\n\t"                         \
         "spm\n\t"                                \
-        : "=m" (__SPM_REG),                      \
-          "=m" (RAMPZ)                           \
-        : "r" ((uint8_t)__BOOT_PAGE_ERASE),      \
+        :                                        \
+        : "i" (_SFR_MEM_ADDR(__SPM_REG)),        \
+          "i" (_SFR_MEM_ADDR(RAMPZ)),            \
+          "r" ((uint8_t)__BOOT_PAGE_ERASE),      \
           "r" ((uint32_t)address)                \
         : "r30", "r31"                           \
     );                                           \
@@ -287,8 +295,9 @@
         "movw r30, %2\n\t"                       \
         "sts %0, %1\n\t"                         \
         "spm\n\t"                                \
-        : "=m" (__SPM_REG)                       \
-        : "r" ((uint8_t)__BOOT_PAGE_WRITE),      \
+        :                                        \
+        : "i" (_SFR_MEM_ADDR(__SPM_REG)),        \
+          "r" ((uint8_t)__BOOT_PAGE_WRITE),      \
           "r" ((uint16_t)address)                \
         : "r30", "r31"                           \
     );                                           \
@@ -303,8 +312,9 @@
         "spm\n\t"                                \
         ".word 0xffff\n\t"                       \
         "nop\n\t"                                \
-        : "=m" (__SPM_REG)                       \
-        : "r" ((uint8_t)__BOOT_PAGE_WRITE),      \
+        :                                        \
+        : "i" (_SFR_MEM_ADDR(__SPM_REG)),        \
+          "r" ((uint8_t)__BOOT_PAGE_WRITE),      \
           "r" ((uint16_t)address)                \
         : "r30", "r31"                           \
     );                                           \
@@ -318,9 +328,10 @@
         "sts %1, %C3\n\t"                        \
         "sts %0, %2\n\t"                         \
         "spm\n\t"                                \
-        : "=m" (__SPM_REG),                      \
-          "=m" (RAMPZ)                           \
-        : "r" ((uint8_t)__BOOT_PAGE_WRITE),      \
+        :                                        \
+        : "i" (_SFR_MEM_ADDR(__SPM_REG)),        \
+          "i" (_SFR_MEM_ADDR(RAMPZ)),            \
+          "r" ((uint8_t)__BOOT_PAGE_WRITE),      \
           "r" ((uint32_t)address)                \
         : "r30", "r31"                           \
     );                                           \
@@ -332,8 +343,9 @@
     (                                            \
         "sts %0, %1\n\t"                         \
         "spm\n\t"                                \
-        : "=m" (__SPM_REG)                       \
-        : "r" ((uint8_t)__BOOT_RWW_ENABLE)       \
+        :                                        \
+        : "i" (_SFR_MEM_ADDR(__SPM_REG)),        \
+          "r" ((uint8_t)__BOOT_RWW_ENABLE)       \
     );                                           \
 })
 
@@ -345,8 +357,9 @@
         "spm\n\t"                                \
         ".word 0xffff\n\t"                       \
         "nop\n\t"                                \
-        : "=m" (__SPM_REG)                       \
-        : "r" ((uint8_t)__BOOT_RWW_ENABLE)       \
+        :                                        \
+        : "i" (_SFR_MEM_ADDR(__SPM_REG)),        \
+          "r" ((uint8_t)__BOOT_RWW_ENABLE)       \
     );                                           \
 })
 
@@ -377,8 +390,9 @@
         "mov r0, %2\n\t"                                   \
         "sts %0, %1\n\t"                                   \
         "spm\n\t"                                          \
-        : "=m" (__SPM_REG)                                 \
-        : "r" ((uint8_t)__BOOT_LOCK_BITS_SET),             \
+        :                                                  \
+        : "i" (_SFR_MEM_ADDR(__SPM_REG)),                  \
+          "r" ((uint8_t)__BOOT_LOCK_BITS_SET),             \
           "r" (value)                                      \
         : "r0", "r30", "r31"                               \
     );                                                     \
@@ -396,13 +410,84 @@
         "spm\n\t"                                          \
         ".word 0xffff\n\t"                                 \
         "nop\n\t"                                          \
-        : "=m" (__SPM_REG)                                 \
-        : "r" ((uint8_t)__BOOT_LOCK_BITS_SET),       \
+        :                                                  \
+        : "i" (_SFR_MEM_ADDR(__SPM_REG)),                  \
+          "r" ((uint8_t)__BOOT_LOCK_BITS_SET),             \
           "r" (value)                                      \
         : "r0", "r30", "r31"                               \
     );                                                     \
 })
 
+/*
+   Reading lock and fuse bits:
+
+     Similarly to writing the lock bits above, set BLBSET and SPMEN bits
+     in __SPMREG, and then (within four clock cycles) issue an LPM
+     instruction.
+
+     Z address:       contents:
+     0x0000           low fuse bits
+     0x0001           lock bits
+     0x0002           extended fuse bits
+     0x0003           high fuse bits
+
+     Sounds confusing, doesn't it?
+
+     Unlike the macros in pgmspace.h, no need to care for non-enhanced
+     cores here as these old cores do not provide SPM support anyway.
+ */
+
+/** \ingroup avr_boot
+    \def GET_LOW_FUSE_BITS
+    address to read the low fuse bits, using boot_lock_fuse_bits_get
+ */
+#define GET_LOW_FUSE_BITS           (0x0000)
+/** \ingroup avr_boot
+    \def GET_LOCK_BITS
+    address to read the lock bits, using boot_lock_fuse_bits_get
+ */
+#define GET_LOCK_BITS               (0x0001)
+/** \ingroup avr_boot
+    \def GET_EXTENDED_FUSE_BITS
+    address to read the extended fuse bits, using boot_lock_fuse_bits_get
+ */
+#define GET_EXTENDED_FUSE_BITS      (0x0002)
+/** \ingroup avr_boot
+    \def GET_HIGH_FUSE_BITS
+    address to read the high fuse bits, using boot_lock_fuse_bits_get
+ */
+#define GET_HIGH_FUSE_BITS          (0x0003)
+
+/** \ingroup avr_boot
+    \def boot_lock_fuse_bits_get(address)
+
+    Read the lock or fuse bits at \c address.
+
+    Parameter \c address can be any of GET_LOW_FUSE_BITS,
+    GET_LOCK_BITS, GET_EXTENDED_FUSE_BITS, or GET_HIGH_FUSE_BITS.
+
+    \note The lock and fuse bits returned are the physical values,
+    i.e. a bit returned as 0 means the corresponding fuse or lock bit
+    is programmed.
+ */
+#define boot_lock_fuse_bits_get(address)                   \
+(__extension__({                                           \
+    uint8_t __result;                                      \
+    __asm__ __volatile__                                   \
+    (                                                      \
+        "ldi r30, %3\n\t"                                  \
+        "ldi r31, 0\n\t"                                   \
+        "sts %1, %2\n\t"                                   \
+        "lpm %0, Z\n\t"                                    \
+        : "=r" (__result)                                  \
+        : "i" (_SFR_MEM_ADDR(__SPM_REG)),                  \
+          "r" ((uint8_t)__BOOT_LOCK_BITS_SET),             \
+          "M" (address)                                    \
+        : "r0", "r30", "r31"                               \
+    );                                                     \
+    __result;                                              \
+}))
+
 /** \ingroup avr_boot
     \def boot_page_fill(address, data)
 
@@ -443,6 +528,7 @@
     \param lock_bits A mask of which Boot Loader Lock Bits to set.
 
     \note In this context, a 'set bit' will be written to a zero value.
+    Note also that only BLBxx bits can be programmed by this command.
 
     For example, to disallow the SPM instruction from writing to the Boot
     Loader memory section of flash, you would use this macro as such:
@@ -451,11 +537,9 @@
     boot_lock_bits_set (_BV (BLB12));
     \endcode
 
-    And to remove any SPM restrictions, you would do this:
-
-    \code
-    boot_lock_bits_set (0);
-    \endcode */
+    \note Like any lock bits, the Boot Loader Lock Bits, once set,
+    cannot be cleared again except by a chip erase which will in turn
+    also erase the boot loader itself. */
 
 /* Normal versions of the macros use 16-bit addresses.
    Extended versions of the macros use 32-bit addresses.
@@ -475,7 +559,7 @@
 #define boot_rww_enable()             __boot_rww_enable_alternate()
 #define boot_lock_bits_set(lock_bits) __boot_lock_bits_set_alternate(lock_bits)
 
-#elif (FLASHEND > USHRT_MAX) && !defined(__USING_MINT8)
+#elif (FLASHEND > USHRT_MAX)
 
 /* Extended: >16 bit address */
 #define boot_page_fill(address, data) __boot_page_fill_extended(address, data)
@@ -495,51 +579,64 @@
 
 #endif
 
-#define __boot_eeprom_spm_safe(func, address, data) \
-do { \
-    boot_spm_busy_wait();                       \
-    eeprom_busy_wait();                         \
-    func (address, data);                       \
-} while (0)
-
 /** \ingroup avr_boot
 
     Same as boot_page_fill() except it waits for eeprom and spm operations to
     complete before filling the page. */
 
 #define boot_page_fill_safe(address, data) \
-    __boot_eeprom_spm_safe (boot_page_fill, address, data)
+do { \
+    boot_spm_busy_wait();                       \
+    eeprom_busy_wait();                         \
+    boot_page_fill(address, data);              \
+} while (0)
 
 /** \ingroup avr_boot
 
     Same as boot_page_erase() except it waits for eeprom and spm operations to
     complete before erasing the page. */
 
-#define boot_page_erase_safe(address, data) \
-    __boot_eeprom_spm_safe (boot_page_erase, address, data)
+#define boot_page_erase_safe(address) \
+do { \
+    boot_spm_busy_wait();                       \
+    eeprom_busy_wait();                         \
+    boot_page_erase (address);                  \
+} while (0)
 
 /** \ingroup avr_boot
 
     Same as boot_page_write() except it waits for eeprom and spm operations to
     complete before writing the page. */
 
-#define boot_page_write_safe(address, data) \
-    __boot_eeprom_spm_safe (boot_page_wrte, address, data)
+#define boot_page_write_safe(address) \
+do { \
+    boot_spm_busy_wait();                       \
+    eeprom_busy_wait();                         \
+    boot_page_write (address);                  \
+} while (0)
 
 /** \ingroup avr_boot
 
     Same as boot_rww_enable() except waits for eeprom and spm operations to
     complete before enabling the RWW mameory. */
 
-#define boot_rww_enable_safe(address, data) \
-    __boot_eeprom_spm_safe (boot_rww_enable, address, data)
+#define boot_rww_enable_safe() \
+do { \
+    boot_spm_busy_wait();                       \
+    eeprom_busy_wait();                         \
+    boot_rww_enable();                          \
+} while (0)
 
 /** \ingroup avr_boot
 
     Same as boot_lock_bits_set() except waits for eeprom and spm operations to
     complete before setting the lock bits. */
 
-#define boot_lock_bits_set_safe(address, data) \
-    __boot_eeprom_spm_safe (boot_lock_bits_set, address, data)
+#define boot_lock_bits_set_safe(lock_bits) \
+do { \
+    boot_spm_busy_wait();                       \
+    eeprom_busy_wait();                         \
+    boot_lock_bits_set (lock_bits);             \
+} while (0)
 
 #endif /* _AVR_BOOT_H_ */
