@@ -28,7 +28,7 @@
   ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
   POSSIBILITY OF SUCH DAMAGE. */
 
-/* $Id: iom128.h,v 1.14 2004/12/30 18:45:13 arcanum Exp $ */
+/* $Id: iom128.h,v 1.18 2005/10/30 22:11:23 joerg_wunsch Exp $ */
 
 /* avr/iom128.h - defines for ATmega128 
 
@@ -139,7 +139,16 @@
 /* Data Register, Port A */
 #define PORTA     _SFR_IO8(0x1B)
 
-/* 0x1C..0x1F EEPROM */
+/* EEPROM Control Register */
+#define EECR      _SFR_IO8(0x1C)
+
+/* EEPROM Data Register */
+#define EEDR      _SFR_IO8(0x1D)
+
+/* EEPROM Address Register */
+#define EEAR      _SFR_IO16(0x1E)
+#define EEARL     _SFR_IO8(0x1E)
+#define EEARH     _SFR_IO8(0x1F)
 
 /* Special Function I/O Register */
 #define SFIOR     _SFR_IO8(0x20)
@@ -351,46 +360,147 @@
 
 /* Interrupt vectors */
 
-#define SIG_INTERRUPT0          _VECTOR(1)
-#define SIG_INTERRUPT1          _VECTOR(2)
-#define SIG_INTERRUPT2          _VECTOR(3)
-#define SIG_INTERRUPT3          _VECTOR(4)
-#define SIG_INTERRUPT4          _VECTOR(5)
-#define SIG_INTERRUPT5          _VECTOR(6)
-#define SIG_INTERRUPT6          _VECTOR(7)
-#define SIG_INTERRUPT7          _VECTOR(8)
-#define SIG_OUTPUT_COMPARE2     _VECTOR(9)
-#define SIG_OVERFLOW2           _VECTOR(10)
-#define SIG_INPUT_CAPTURE1      _VECTOR(11)
-#define SIG_OUTPUT_COMPARE1A    _VECTOR(12)
-#define SIG_OUTPUT_COMPARE1B    _VECTOR(13)
-#define SIG_OVERFLOW1           _VECTOR(14)
-#define SIG_OUTPUT_COMPARE0     _VECTOR(15)
-#define SIG_OVERFLOW0           _VECTOR(16)
-#define SIG_SPI                 _VECTOR(17)
-#define SIG_USART0_RECV         _VECTOR(18)
-#define SIG_UART0_RECV          _VECTOR(18) /* Keep for compatibility */
-#define SIG_USART0_DATA         _VECTOR(19)
-#define SIG_UART0_DATA          _VECTOR(19) /* Keep for compatibility */
-#define SIG_USART0_TRANS        _VECTOR(20)
-#define SIG_UART0_TRANS         _VECTOR(20) /* Keep for compatibility */
-#define SIG_ADC                 _VECTOR(21)
-#define SIG_EEPROM_READY        _VECTOR(22)
-#define SIG_COMPARATOR          _VECTOR(23)
-#define SIG_OUTPUT_COMPARE1C    _VECTOR(24)
-#define SIG_INPUT_CAPTURE3      _VECTOR(25)
-#define SIG_OUTPUT_COMPARE3A    _VECTOR(26)
-#define SIG_OUTPUT_COMPARE3B    _VECTOR(27)
-#define SIG_OUTPUT_COMPARE3C    _VECTOR(28)
-#define SIG_OVERFLOW3           _VECTOR(29)
-#define SIG_USART1_RECV         _VECTOR(30)
-#define SIG_UART1_RECV          _VECTOR(30) /* Keep for compatibility */
-#define SIG_USART1_DATA         _VECTOR(31)
-#define SIG_UART1_DATA          _VECTOR(31) /* Keep for compatibility */
-#define SIG_USART1_TRANS        _VECTOR(32)
-#define SIG_UART1_TRANS         _VECTOR(32) /* Keep for compatibility */
-#define SIG_2WIRE_SERIAL        _VECTOR(33)
-#define SIG_SPM_READY           _VECTOR(34)
+/* External Interrupt Request 0 */
+#define INT0_vect			_VECTOR(1)
+#define SIG_INTERRUPT0			_VECTOR(1)
+
+/* External Interrupt Request 1 */
+#define INT1_vect			_VECTOR(2)
+#define SIG_INTERRUPT1			_VECTOR(2)
+
+/* External Interrupt Request 2 */
+#define INT2_vect			_VECTOR(3)
+#define SIG_INTERRUPT2			_VECTOR(3)
+
+/* External Interrupt Request 3 */
+#define INT3_vect			_VECTOR(4)
+#define SIG_INTERRUPT3			_VECTOR(4)
+
+/* External Interrupt Request 4 */
+#define INT4_vect			_VECTOR(5)
+#define SIG_INTERRUPT4			_VECTOR(5)
+
+/* External Interrupt Request 5 */
+#define INT5_vect			_VECTOR(6)
+#define SIG_INTERRUPT5			_VECTOR(6)
+
+/* External Interrupt Request 6 */
+#define INT6_vect			_VECTOR(7)
+#define SIG_INTERRUPT6			_VECTOR(7)
+
+/* External Interrupt Request 7 */
+#define INT7_vect			_VECTOR(8)
+#define SIG_INTERRUPT7			_VECTOR(8)
+
+/* Timer/Counter2 Compare Match */
+#define TIMER2_COMP_vect		_VECTOR(9)
+#define SIG_OUTPUT_COMPARE2		_VECTOR(9)
+
+/* Timer/Counter2 Overflow */
+#define TIMER2_OVF_vect			_VECTOR(10)
+#define SIG_OVERFLOW2			_VECTOR(10)
+
+/* Timer/Counter1 Capture Event */
+#define TIMER1_CAPT_vect		_VECTOR(11)
+#define SIG_INPUT_CAPTURE1		_VECTOR(11)
+
+/* Timer/Counter1 Compare Match A */
+#define TIMER1_COMPA_vect		_VECTOR(12)
+#define SIG_OUTPUT_COMPARE1A		_VECTOR(12)
+
+/* Timer/Counter Compare Match B */
+#define TIMER1_COMPB_vect		_VECTOR(13)
+#define SIG_OUTPUT_COMPARE1B		_VECTOR(13)
+
+/* Timer/Counter1 Overflow */
+#define TIMER1_OVF_vect			_VECTOR(14)
+#define SIG_OVERFLOW1			_VECTOR(14)
+
+/* Timer/Counter0 Compare Match */
+#define TIMER0_COMP_vect		_VECTOR(15)
+#define SIG_OUTPUT_COMPARE0		_VECTOR(15)
+
+/* Timer/Counter0 Overflow */
+#define TIMER0_OVF_vect			_VECTOR(16)
+#define SIG_OVERFLOW0			_VECTOR(16)
+
+/* SPI Serial Transfer Complete */
+#define SPI_STC_vect			_VECTOR(17)
+#define SIG_SPI				_VECTOR(17)
+
+/* USART0, Rx Complete */
+#define USART0_RX_vect			_VECTOR(18)
+#define SIG_USART0_RECV			_VECTOR(18)
+#define SIG_UART0_RECV			_VECTOR(18)
+
+/* USART0 Data Register Empty */
+#define USART0_UDRE_vect		_VECTOR(19)
+#define SIG_USART0_DATA			_VECTOR(19)
+#define SIG_UART0_DATA			_VECTOR(19)
+
+/* USART0, Tx Complete */
+#define USART0_TX_vect			_VECTOR(20)
+#define SIG_USART0_TRANS		_VECTOR(20)
+#define SIG_UART0_TRANS			_VECTOR(20)
+
+/* ADC Conversion Complete */
+#define ADC_vect			_VECTOR(21)
+#define SIG_ADC				_VECTOR(21)
+
+/* EEPROM Ready */
+#define EE_READY_vect			_VECTOR(22)
+#define SIG_EEPROM_READY		_VECTOR(22)
+
+/* Analog Comparator */
+#define ANALOG_COMP_vect		_VECTOR(23)
+#define SIG_COMPARATOR			_VECTOR(23)
+
+/* Timer/Counter1 Compare Match C */
+#define TIMER1_COMPC_vect		_VECTOR(24)
+#define SIG_OUTPUT_COMPARE1C		_VECTOR(24)
+
+/* Timer/Counter3 Capture Event */
+#define TIMER3_CAPT_vect		_VECTOR(25)
+#define SIG_INPUT_CAPTURE3		_VECTOR(25)
+
+/* Timer/Counter3 Compare Match A */
+#define TIMER3_COMPA_vect		_VECTOR(26)
+#define SIG_OUTPUT_COMPARE3A		_VECTOR(26)
+
+/* Timer/Counter3 Compare Match B */
+#define TIMER3_COMPB_vect		_VECTOR(27)
+#define SIG_OUTPUT_COMPARE3B		_VECTOR(27)
+
+/* Timer/Counter3 Compare Match C */
+#define TIMER3_COMPC_vect		_VECTOR(28)
+#define SIG_OUTPUT_COMPARE3C		_VECTOR(28)
+
+/* Timer/Counter3 Overflow */
+#define TIMER3_OVF_vect			_VECTOR(29)
+#define SIG_OVERFLOW3			_VECTOR(29)
+
+/* USART1, Rx Complete */
+#define USART1_RX_vect			_VECTOR(30)
+#define SIG_USART1_RECV			_VECTOR(30)
+#define SIG_UART1_RECV			_VECTOR(30)
+
+/* USART1, Data Register Empty */
+#define USART1_UDRE_vect		_VECTOR(31)
+#define SIG_USART1_DATA			_VECTOR(31)
+#define SIG_UART1_DATA			_VECTOR(31)
+
+/* USART1, Tx Complete */
+#define USART1_TX_vect			_VECTOR(32)
+#define SIG_USART1_TRANS		_VECTOR(32)
+#define SIG_UART1_TRANS			_VECTOR(32)
+
+/* 2-wire Serial Interface */
+#define TWI_vect			_VECTOR(33)
+#define SIG_2WIRE_SERIAL		_VECTOR(33)
+
+/* Store Program Memory Read */
+#define SPM_READY_vect			_VECTOR(34)
+#define SIG_SPM_READY			_VECTOR(34)
 
 #define _VECTORS_SIZE 140
 
@@ -683,9 +793,14 @@
 #define    WDP1         1
 #define    WDP0         0
 
+/* 
+   The ADHSM bit has been removed from all documentation, 
+   as being not needed at all since the comparator has proven 
+   to be fast enough even without feeding it more power.
+*/
+
 /* Special Function I/O Register - SFIOR */
 #define    TSM          7
-#define    ADHSM        4
 #define    ACME         3
 #define    PUD          2
 #define    PSR0         1
@@ -1025,6 +1140,12 @@
 #define    PING1        1
 #define    PING0        0
 
+/* EEPROM Control Register */
+#define    EERIE        3
+#define    EEMWE        2
+#define    EEWE         1
+#define    EERE         0
+
 /* Constants */
 #define SPM_PAGESIZE 256
 #define    RAMEND   0x10FF     /* Last On-Chip SRAM Location */
